package com.ruoyi.bst.customer.service.impl;

import java.time.LocalDateTime;
import java.util.Collections;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.support.TransactionTemplate;

import com.ruoyi.bst.customer.domain.Customer;
import com.ruoyi.bst.customer.domain.CustomerQuery;
import com.ruoyi.bst.customer.domain.CustomerVO;
import com.ruoyi.bst.customer.domain.dto.CustomerAddDTO;
import com.ruoyi.bst.customer.mapper.CustomerMapper;
import com.ruoyi.bst.customer.service.CustomerService;
import com.ruoyi.bst.customer.service.CustomerValidator;
import com.ruoyi.bst.customerFollow.service.CustomerFollowService;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.common.utils.SnowFlakeUtil;
import com.ruoyi.common.utils.collection.CollectionUtils;
import com.ruoyi.common.vo.LocalDateIntegerVO;
import com.ruoyi.common.vo.StringIntegerVO;

/**
 * 客户Service业务层处理
 *
 * @author ruoyi
 * @date 2025-01-22
 */
@Service
public class CustomerServiceImpl implements CustomerService
{
    @Autowired
    private CustomerMapper customerMapper;

    @Autowired
    private TransactionTemplate transactionTemplate;

    @Autowired
    private CustomerValidator customerValidator;

    @Autowired
    private CustomerFollowService customerFollowService;

    /**
     * 查询客户
     *
     * @param id 客户主键
     * @return 客户
     */
    @Override
    public CustomerVO selectCustomerById(Long id)
    {
        return customerMapper.selectCustomerById(id);
    }

    /**
     * 查询客户列表
     *
     * @param customer 客户
     * @return 客户
     */
    @Override
    public List<CustomerVO> selectCustomerList(CustomerQuery customer)
    {
        return customerMapper.selectCustomerList(customer);
    }

    /**
     * 新增客户
     *
     * @param customer 客户
     * @return 结果
     */
    @Override
    public int insertCustomer(CustomerAddDTO customer)
    {
        customer.setCode(String.valueOf(SnowFlakeUtil.newId()));
        customer.setCreateTime(DateUtils.getNowDate());
        customer.setCreateId(SecurityUtils.getUserId());

        Integer result = transactionTemplate.execute(status -> {
            int insert = customerMapper.insertCustomer(customer);
            if (insert > 0) {

                // 若跟进记录不为空，则插入跟进记录
                if (customer.getFollow() != null) {
                    customer.getFollow().setCustomerId(customer.getId());
                    int insertFollow = customerFollowService.insertCustomerFollow(customer.getFollow());
                    ServiceUtil.assertion(insertFollow != 1, "新增跟进记录失败");
                }

                CustomerVO vo = selectCustomerById(customer.getId());
                customerValidator.validate(vo);
            }
            return insert;
        });

        return result;
    }

    /**
     * 修改客户
     *
     * @param customer 客户
     * @return 结果
     */
    @Override
    public int updateCustomer(Customer customer)
    {
        customer.setUpdateTime(DateUtils.getNowDate());

        Integer result = transactionTemplate.execute(status -> {
            int update = customerMapper.updateCustomer(customer);
            if (update > 0) {
                CustomerVO vo = selectCustomerById(customer.getId());
                customerValidator.validate(vo);
            }
            return update;
        });

        return result;
    }

    @Override
    public int selectCount(CustomerQuery query) {
        return customerMapper.selectCount(query);
    }

    @Override
    public List<CustomerVO> selectCustomerListByIds(List<Long> ids) {
        if (CollectionUtils.isEmptyElement(ids)) {
            return Collections.emptyList();
        }
        CustomerQuery query = new CustomerQuery();
        query.setIds(ids);
        return customerMapper.selectCustomerList(query);
    }

    @Override
    public List<StringIntegerVO> selectCountGroupByStatus(CustomerQuery customerQuery) {
        return customerMapper.selectCountGroupByStatus(customerQuery);
    }

    @Override
    public int updateFollowTime(Long customerId, LocalDateTime lastFollowTime) {
        if (customerId == null) {
            return 0;
        }
        return customerMapper.updateFollowTime(customerId, lastFollowTime);
    }

    @Override
    public int logicDel(List<Long> ids) {
        if (CollectionUtils.isEmptyElement(ids)) {
            return 0;
        }
        return customerMapper.logicDel(ids);
    }

    @Override
    public List<LocalDateIntegerVO> selectDailyCreateCount(CustomerQuery query) {
        return customerMapper.selectDailyCreateCount(query);
    }
}
