package com.ruoyi.bst.task.service.impl;

import java.time.LocalDateTime;
import java.util.Collections;
import java.util.List;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.support.TransactionTemplate;

import com.ruoyi.bst.message.domain.Message;
import com.ruoyi.bst.message.service.MessageConverter;
import com.ruoyi.bst.message.service.MessageService;
import com.ruoyi.bst.task.domain.Task;
import com.ruoyi.bst.task.domain.TaskQuery;
import com.ruoyi.bst.task.domain.TaskVO;
import com.ruoyi.bst.task.domain.dto.TaskCancelDTO;
import com.ruoyi.bst.task.domain.dto.TaskSubmitDTO;
import com.ruoyi.bst.task.domain.enums.TaskStatus;
import com.ruoyi.bst.task.mapper.TaskMapper;
import com.ruoyi.bst.task.service.TaskAssembler;
import com.ruoyi.bst.task.service.TaskService;
import com.ruoyi.bst.task.service.TaskValidator;
import com.ruoyi.bst.taskMember.domain.TaskMember;
import com.ruoyi.bst.taskMember.domain.TaskMemberVO;
import com.ruoyi.bst.taskMember.domain.enums.TaskMemberStatus;
import com.ruoyi.bst.taskMember.service.TaskMemberService;
import com.ruoyi.bst.taskSubmit.domain.TaskSubmit;
import com.ruoyi.bst.taskSubmit.domain.TaskSubmitVO;
import com.ruoyi.bst.taskSubmit.service.TaskSubmitAssembler;
import com.ruoyi.bst.taskSubmit.service.TaskSubmitConverter;
import com.ruoyi.bst.taskSubmit.service.TaskSubmitService;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.common.utils.collection.CollectionUtils;
import com.ruoyi.common.utils.collection.DiffListVO;
import com.ruoyi.common.vo.LongIntegerVO;
import com.ruoyi.common.vo.StringIntegerVO;

import lombok.extern.slf4j.Slf4j;

/**
 * 任务Service业务层处理
 *
 * @author ruoyi
 * @date 2025-01-22
 */
@Service
@Slf4j
public class TaskServiceImpl implements TaskService
{
    @Autowired
    private TaskMapper taskMapper;

    @Autowired
    private TransactionTemplate transactionTemplate;

    @Autowired
    private TaskSubmitConverter taskSubmitConverter;

    @Autowired
    private TaskSubmitService taskSubmitService;

    @Autowired
    private TaskValidator taskValidator;

    @Autowired
    private MessageService MessageService;

    @Autowired
    private MessageConverter messageConverter;

    @Autowired
    private TaskMemberService taskMemberService;

    @Autowired
    private TaskAssembler taskAssembler;

    @Autowired
    private TaskSubmitAssembler taskSubmitAssembler;

    @Autowired
    private ScheduledExecutorService scheduledExecutorService;

    /**
     * 查询任务
     *
     * @param id 任务主键
     * @return 任务
     */
    @Override
    public TaskVO selectTaskById(Long id)
    {
        if (id == null) {
            return null;
        }
        return taskMapper.selectTaskById(id);
    }

    /**
     * 查询任务列表
     *
     * @param task 任务
     * @return 任务
     */
    @Override
    public List<TaskVO> selectTaskList(TaskQuery task)
    {
        return taskMapper.selectTaskList(task);
    }

    /**
     * 新增任务
     *
     * @param task 任务
     * @return 结果
     */
    @Override
    public int insertTask(TaskVO task)
    {
        task.setCreateTime(DateUtils.getNowDate());
        Integer result = transactionTemplate.execute(status -> {
            int rows = taskMapper.insertTask(task);
            if (rows > 0) {

                // 保存成员
                this.saveMemberList(task);

                // 后校验
                TaskVO vo = this.selectTaskById(task.getId());
                taskAssembler.assembleMemberList(vo);
                taskAssembler.assembleProjectMemberList(vo);
                taskValidator.validate(vo);

                // 保存并发送消息
                List<Message> messages = messageConverter.toPoByTaskCreate(vo);
                int messageRows = MessageService.batchInsert(messages);
                ServiceUtil.assertion(messageRows != messages.size(), "新增消息失败");
            }
            return rows;
        });

        return result == null ? 0 : result;
    }

    /**
     * 修改任务
     *
     * @param task 任务
     * @return 结果
     */
    @Override
    public int updateTask(TaskVO task)
    {
        task.setUpdateTime(DateUtils.getNowDate());
        Integer result = transactionTemplate.execute(status -> {
            int rows = taskMapper.updateTask(task);

            if (rows > 0) {

                // 保存成员
                this.saveMemberList(task);

                // 后校验
                TaskVO vo = this.selectTaskById(task.getId());
                taskAssembler.assembleMemberList(vo);
                taskAssembler.assembleProjectMemberList(vo);
                taskValidator.validate(vo);

                // 保存并发送消息
                List<Message> messages = messageConverter.toPoByTaskUpdate(vo);
                int messageRows = MessageService.batchInsert(messages);
                ServiceUtil.assertion(messageRows != messages.size(), "保存消息失败");

            }

            return rows;
        });

        return result == null ? 0 : result;
    }

    private int saveMemberList(TaskVO task) {
        if (task.getMemberList() == null) {
            return 0;
        }
        // 设置任务ID
        task.getMemberList().forEach(member -> {
            member.setTaskId(task.getId());
        });

        // 查询旧成员
        List<TaskMemberVO> oldList = taskMemberService.selectListByTaskId(task.getId());

        // 根据用户ID转为差异列表
        DiffListVO<TaskMemberVO, TaskMemberVO> diff = CollectionUtils.convertToDiffList(task.getMemberList(), oldList, TaskMember::getUserId);

        Integer result = transactionTemplate.execute(status -> {
            int rows = 0;
            // 新增
            if (diff.getAddCount() > 0) {
                int insert = taskMemberService.batchInsert(diff.getAddList());
                ServiceUtil.assertion(insert != diff.getAddCount(), "新增负责人失败");
                rows += insert;
            }
            // 删除
            if (diff.getDelCount() > 0) {
                int delete = taskMemberService.batchDelete(diff.getDelList());
                ServiceUtil.assertion(delete != diff.getDelCount(), "删除负责人失败");
                rows += delete;
            }
            return rows;
        });

        return result == null ? 0 : result;

    }

    /**
     * 批量删除任务
     *
     * @param ids 需要删除的任务主键
     * @return 结果
     */
    @Override
    public int logicDel(List<Long> ids)
    {
        return taskMapper.logicDel(ids);
    }

    @Override
    public List<StringIntegerVO> selectCountGroupByStatus(TaskQuery query) {
        return taskMapper.selectCountGroupByStatus(query);
    }

    @Override
    public int selectCount(TaskQuery query) {
        return taskMapper.selectCount(query);
    }

    @Override
    public List<LongIntegerVO> selectCountGroupByProject(TaskQuery query) {
        return taskMapper.selectCountGroupByProject(query);
    }

    @Override
    public int submitTask(TaskSubmitDTO dto) {
        // 校验
        ServiceUtil.assertion(dto == null, "参数不允许为空");
        ServiceUtil.assertion(dto.getId() == null, "任务ID不允许为空");
        TaskVO old = this.selectTaskById(dto.getId());
        ServiceUtil.assertion(old == null, "ID为%s的任务不存在", dto.getId());
        ServiceUtil.assertion(!TaskStatus.canSubmit().contains(old.getStatus()), "ID为%s的任务当前状态不允许提交", dto.getId());

        Integer result = transactionTemplate.execute(status -> {
            // 新增提交记录
            TaskSubmit submit = taskSubmitConverter.toPo(dto);
            int insertSubmit = taskSubmitService.insertTaskSubmit(submit);
            ServiceUtil.assertion(insertSubmit != 1, "新增提交记录失败");

            // 查询提交记录
            TaskSubmitVO submitVo = taskSubmitService.selectTaskSubmitById(submit.getId());

            // 尝试更新成员的状态
            int submitRows = taskMemberService.submit(submitVo.getTaskId(), submitVo.getUserId(), submitVo.getId());
            ServiceUtil.assertion(submitRows != 1, "更新成员状态失败");

            // 保存并发送消息
            taskSubmitAssembler.assembleTaskMemberList(Collections.singletonList(submitVo));
            List<Message> messages = messageConverter.toPoByTaskSubmit(submitVo);
            int messageRows = MessageService.batchInsert(messages);
            ServiceUtil.assertion(messageRows != messages.size(), "保存消息失败");

            return insertSubmit;
        });

        // 尝试更新状态为结束
        if (result != null && result > 0) {
            scheduledExecutorService.schedule(() -> {
                this.passTask(dto.getId());
            }, 0, TimeUnit.SECONDS);
        }

        return result == null ? 0 : result;
    }

    @Override
    public int updateByQuery(Task data, TaskQuery query) {
        return taskMapper.updateByQuery(data, query);
    }


    @Override
    public int passTask(Long id) {
        // 校验
        ServiceUtil.assertion(id == null, "任务ID不允许为空");

        TaskVO old = this.selectTaskById(id);
        ServiceUtil.assertion(old == null, "ID为%s的任务不存在", id);
        ServiceUtil.assertion(!TaskStatus.canPass().contains(old.getStatus()), "ID为%s的任务当前状态不允许完成", id);

        taskAssembler.assembleMemberList(Collections.singletonList(old));

        // 查询提交记录，判断是否所有负责人都已经通过了
        if (old.getMemberList().size() > 0) {
            List<TaskMemberVO> unSubmitMembers = old.getMemberList().stream().filter(member -> TaskMemberStatus.unSubmit().contains(member.getStatus())).collect(Collectors.toList());
            ServiceUtil.assertion(unSubmitMembers.size() > 0,
                "ID为%s的任务当前状态不允许完成,还有%s个负责人未提交。分别为:%s",
                id,
                unSubmitMembers.size(),
                unSubmitMembers.stream().map(TaskMemberVO::getUserName).collect(Collectors.joining(","))
            );
        }

        // 更新任务状态
        Integer result = transactionTemplate.execute(status -> {

            Task data = new Task();
            data.setPassTime(LocalDateTime.now());
            data.setStatus(TaskStatus.PASS.getStatus());
            TaskQuery query = new TaskQuery();
            query.setId(id);
            query.setStatusList(TaskStatus.canPass());
            int rows =this.updateByQuery(data, query);

            if (rows > 0) {
                // 保存并发送消息
                TaskVO vo = this.selectTaskById(id);
                List<Message> messages = messageConverter.toPoByTaskPass(vo);
                int messageRows = MessageService.batchInsert(messages);
                ServiceUtil.assertion(messageRows != messages.size(), "保存消息失败");
            }

            return rows;
        });

        return result == null ? 0 : result;
    }

    @Override
    public int cancelTask(TaskCancelDTO dto) {

        // 校验
        ServiceUtil.assertion(dto == null, "参数不允许为空");
        ServiceUtil.assertion(dto.getId() == null, "任务ID不允许为空");

        TaskVO old = this.selectTaskById(dto.getId());
        ServiceUtil.assertion(old == null, "ID为%s的任务不存在", dto.getId());
        ServiceUtil.assertion(!TaskStatus.canCancel().contains(old.getStatus()), "ID为%s的任务当前状态不允许取消", dto.getId());

        Integer result = transactionTemplate.execute(status -> {
            // 更新任务状态
            Task data = new Task();
            data.setStatus(TaskStatus.CANCEL.getStatus());
            data.setCancelTime(LocalDateTime.now());
            data.setCancelRemark(dto.getCancelRemark());
            data.setCancelUserId(SecurityUtils.getUserId());
            TaskQuery query = new TaskQuery();
            query.setId(dto.getId());
            query.setStatusList(TaskStatus.canCancel());
            int rows = this.updateByQuery(data, query);

            if (rows > 0) {
                // 保存并发送消息
                TaskVO vo = this.selectTaskById(dto.getId());
                taskAssembler.assembleMemberList(vo);
                List<Message> messages = messageConverter.toPoByTaskCancel(vo);
                int messageRows = MessageService.batchInsert(messages);
                ServiceUtil.assertion(messageRows != messages.size(), "保存消息失败");
            }

            return rows;
        });

        return result == null ? 0 : result;
    }

    @Override
    public int startTask(Long taskId) {
        // 校验
        ServiceUtil.assertion(taskId == null, "任务ID不允许为空");
        TaskVO old = this.selectTaskById(taskId);
        ServiceUtil.assertion(old == null, "ID为%s的任务不存在", taskId);
        return this.startTask(old);
    }

    private int startTask(TaskVO task) {
        ServiceUtil.assertion(task == null, "任务不存在");
        ServiceUtil.assertion(!TaskStatus.canStart().contains(task.getStatus()), "ID为%s的任务当前状态不允许开始", task.getId());

        // 更新任务状态
        Task data = new Task();
        data.setStatus(TaskStatus.PROCESSING.getStatus());
        TaskQuery query = new TaskQuery();
        query.setId(task.getId());
        query.setStatusList(TaskStatus.canStart());
        return this.updateByQuery(data, query);
    }

    @Override
    public List<StringIntegerVO> selectCountGroupByType(TaskQuery query) {
        return taskMapper.selectCountGroupByType(query);
    }

    @Override
    public List<TaskVO> selectTaskByIds(List<Long> ids) {
        if (CollectionUtils.isEmptyElement(ids)) {
            return Collections.emptyList();
        }
        TaskQuery query = new TaskQuery();
        query.setIds(ids);
        return selectTaskList(query);
    }

    @Override
    public int receive(Long id, Long userId) {
        if (id == null || userId == null) {
            return 0;
        }

        TaskVO task = this.selectTaskById(id);
        ServiceUtil.assertion(task == null, "ID为%s的任务不存在", id);

        Integer result = transactionTemplate.execute(status -> {
            // 用户尝试接收任务
            int received = taskMemberService.receive(id, userId);
            ServiceUtil.assertion(received != 1, "接收ID为%s的任务失败", id);

            // 若任务状态为待接收，则开始任务
            if (received > 0 && TaskStatus.WAIT_RECEIVE.getStatus().equals(task.getStatus())) {
                int start = this.startTask(task);
                ServiceUtil.assertion(start != 1, "开始ID为%s的任务失败", id);
            }

            return received;
        });

        return result == null ? 0 : result;
    }


}
