package com.ruoyi.bst.task.service.impl;

import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.ruoyi.bst.projectMember.domain.ProjectMemberVO;
import com.ruoyi.bst.task.domain.TaskVO;
import com.ruoyi.bst.task.service.TaskService;
import com.ruoyi.bst.task.service.TaskValidator;
import com.ruoyi.bst.taskMember.domain.TaskMemberVO;
import com.ruoyi.bst.taskMember.service.TaskMemberValidator;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.common.utils.collection.CollectionUtils;

@Service
public class TaskValidatorImpl implements TaskValidator{

    @Autowired
    private TaskService taskService;

    @Autowired
    private TaskMemberValidator taskMemberValidator;

    @Override
    public boolean allowSubmit(Long id, Long userId) {
        if (id == null || userId == null) {
            return false;
        }
        return taskMemberValidator.isMember(id, userId);
    }

    @Override
    public void validate(TaskVO vo) {
        ServiceUtil.assertion(vo == null, "任务不能为空");

        if (vo.getProjectId() != null && CollectionUtils.isNotEmpty(vo.getMemberList())) {
            // 验证任务成员必须是项目成员的子集

            // 获取项目成员的用户ID列表
            List<Long> projectMemberUserIds = CollectionUtils.map(vo.getProjectMemberList(), ProjectMemberVO::getUserId);
            ServiceUtil.assertion(CollectionUtils.isEmptyElement(projectMemberUserIds), "项目成员列表不能为空");

            // 过滤出不存在于项目成员中的任务成员
            List<TaskMemberVO> notInProjectMemberList = vo.getMemberList().stream()
                .filter(member -> !projectMemberUserIds.contains(member.getUserId()))
                .collect(Collectors.toList());
            ServiceUtil.assertion(CollectionUtils.isNotEmptyElement(notInProjectMemberList),
             "任务负责人%s不是项目【%s】的成员", CollectionUtils.map(notInProjectMemberList, TaskMemberVO::getUserName), vo.getProjectName());
        }

    }

    /**
     * 是否允许修改任务
     */
    @Override
    public boolean allowUpdateTask(TaskVO vo) {
        return SecurityUtils.isSysAdmin() || isCreator(vo, SecurityUtils.getUserId());
    }

    @Override
    public boolean allowCancelTask(TaskVO vo) {
        return SecurityUtils.isSysAdmin() || isCreator(vo, SecurityUtils.getUserId());
    }

    @Override
    public boolean allowDelAllTask(List<TaskVO> old) {
        if (SecurityUtils.isSysAdmin()) {
            return true;
        }
        Long userId  = SecurityUtils.getUserId();
        return old.stream().allMatch(vo -> isCreator(vo, userId));
    }

    /**
     * 是否是创建人
     */
    private boolean isCreator(TaskVO vo, Long userId) {
        return vo != null && vo.getCreateId() != null && Objects.equals(vo.getCreateId(), userId);
    }
}
