package com.ruoyi.web.bst;

import java.util.Collections;
import java.util.List;

import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.ruoyi.bst.task.domain.TaskQuery;
import com.ruoyi.bst.task.domain.TaskVO;
import com.ruoyi.bst.task.domain.dto.TaskCancelDTO;
import com.ruoyi.bst.task.domain.dto.TaskSubmitDTO;
import com.ruoyi.bst.task.service.TaskAssembler;
import com.ruoyi.bst.task.service.TaskConverter;
import com.ruoyi.bst.task.service.TaskService;
import com.ruoyi.bst.task.service.TaskValidator;
import com.ruoyi.bst.taskMember.domain.TaskMemberVO;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.constant.RoleConstants;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.core.validate.ValidGroup;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.poi.ExcelUtil;

/**
 * 任务Controller
 *
 * @author ruoyi
 * @date 2025-01-22
 */
@RestController
@RequestMapping("/bst/task")
public class TaskController extends BaseController
{
    @Autowired
    private TaskService taskService;

    @Autowired
    private TaskConverter taskConverter;

    @Autowired
    private TaskValidator taskValidator;

    @Autowired
    private TaskAssembler taskAssembler;

    /**
     * 查询任务列表
     */
    @PreAuthorize("@ss.hasPermi('bst:task:list')")
    @GetMapping("/list")
    public TableDataInfo list(TaskQuery query)
    {
        startPage();
        startOrderBy();
        query.setScope(true);
        List<TaskVO> list = taskService.selectTaskList(query);
        taskAssembler.assembleMemberList(list);
        return getDataTable(list);
    }

    /**
     * 导出任务列表
     */
    @PreAuthorize("@ss.hasPermi('bst:task:export')")
    @Log(title = "任务", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, TaskQuery query)
    {
        List<TaskVO> list = taskService.selectTaskList(query);
        ExcelUtil<TaskVO> util = new ExcelUtil<TaskVO>(TaskVO.class);
        util.exportExcel(response, list, "任务数据");
    }

    /**
     * 获取任务详细信息
     * @param id 任务ID
     * @param receive 是否接收
     */
    @PreAuthorize("@ss.hasPermi('bst:task:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        TaskVO task = taskService.selectTaskById(id);
        List<TaskVO> list = Collections.singletonList(task);
        taskAssembler.assembleSubmitList(list);
        taskAssembler.assembleMemberList(list);
        return success(task);
    }

    /**
     * 新增任务
     */
    @PreAuthorize("@ss.hasPermi('bst:task:add')")
    @Log(title = "任务", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody @Validated(ValidGroup.Create.class) TaskVO task)
    {
        // 若不是管理员，则负责人只能填写自己
        if (!SecurityUtils.hasAnyRole(RoleConstants.ADMIN, RoleConstants.SYS_ADMIN)) {
            task.setMemberList(Collections.singletonList(new TaskMemberVO(getUserId())));
        }
        task = taskConverter.toVoByCreate(task);
        return toAjax(taskService.insertTask(task));
    }

    /**
     * 修改任务
     */
    @PreAuthorize("@ss.hasPermi('bst:task:edit')")
    @Log(title = "任务", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody @Validated(ValidGroup.Update.class) TaskVO task)
    {
        TaskVO old = taskService.selectTaskById(task.getId());
        if (!taskValidator.allowUpdateTask(old)) {
            return error("您无权修改ID为" + task.getId() + "的任务");
        }
        task = taskConverter.toVoByUpdate(task);
        return toAjax(taskService.updateTask(task));
    }

    /**
     * 删除任务
     */
    @PreAuthorize("@ss.hasPermi('bst:task:remove')")
    @Log(title = "任务", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable List<Long> ids)
    {
        List<TaskVO> old = taskService.selectTaskByIds(ids);
        if (!taskValidator.allowDelAllTask(old)) {
            return error("您无权修改ID为" + ids + "的任务");
        }
        return toAjax(taskService.logicDel(ids));
    }

    /**
     * 提交任务
     */
    @PreAuthorize("@ss.hasPermi('bst:task:submit')")
    @Log(title = "任务", businessType = BusinessType.UPDATE)
    @PutMapping("/submit")
    public AjaxResult submit(@RequestBody @Validated TaskSubmitDTO dto) {
        // 校验当前用户能否提交任务
        if (!taskValidator.allowSubmit(dto.getId(), getUserId())) {
            return error("您不是任务的负责人，不能提交任务");
        }
        return toAjax(taskService.submitTask(dto));
    }

    /**
     * 取消任务
     */
    @PreAuthorize("@ss.hasPermi('bst:task:cancel')")
    @Log(title = "任务", businessType = BusinessType.UPDATE)
    @PutMapping("/cancel")
    public AjaxResult cancel(@RequestBody @Validated TaskCancelDTO dto) {
        TaskVO old = taskService.selectTaskById(dto.getId());
        if (!taskValidator.allowCancelTask(old)) {
            return error("您无权取消ID为" + dto.getId() + "的任务");
        }
        return toAjax(taskService.cancelTask(dto));
    }

    /**
     * 通过任务（结束）
     */
    @PreAuthorize("@ss.hasPermi('bst:task:pass')")
    @Log(title = "任务", businessType = BusinessType.UPDATE)
    @PutMapping("/pass/{id}")
    public AjaxResult pass(@PathVariable Long id) {
        return toAjax(taskService.passTask(id));
    }

    /**
     * 接取任务
     */
    @PreAuthorize("@ss.hasPermi('bst:task:receive')")
    @Log(title = "任务", businessType = BusinessType.UPDATE)
    @PutMapping("/receive/{id}")
    public AjaxResult receive(@PathVariable Long id) {
        return toAjax(taskService.receive(id, getUserId()));
    }
}
